==> cvsync-0.24.13.tar.gz: cvsync-0.24.13/rcscan/main.c <==
/*-
 * Copyright (c) 2000-2003 MAEKAWA Masahide <maekawa@cvsync.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <sys/types.h>
#include <sys/mman.h>
#include <sys/uio.h>

#include <stdio.h>
#include <stdlib.h>

#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#include "compat_inttypes.h"
#include "rcslib.h"

boolean rcscan_dump(struct rcslib_file *);
void usage(void);

int
main(int argc, char *argv[])
{
	const char *fname;
	struct rcslib_file *rcs;
	void *addr;
	off_t size;
	uint64_t size64;
	int fd, ch;
	boolean quiet = FALSE;

	while ((ch = getopt(argc, argv, "q")) != -1) {
		switch (ch) {
		case 'q':
			if (quiet) {
				usage();
				/* NOTREACHED */
			}
			quiet = TRUE;
			break;
		default:
			usage();
			/* NOTREACHED */
		}
	}
	argc -= optind;
	argv += optind;

	if (argc == 0) {
		usage();
		/* NOTREACHED */
	}

	while (argc > 0) {
		fname = argv[--argc];

		if ((fd = open(fname, O_RDONLY, 0)) == -1) {
			fprintf(stderr, "%s: %s\n", fname, strerror(errno));
			exit(EXIT_FAILURE);
		}

		if ((size = lseek(fd, 0, SEEK_END)) == (off_t)-1) {
			fprintf(stderr, "%s: %s\n", fname, strerror(errno));
			exit(EXIT_FAILURE);
		}
		size64 = (uint64_t)size;
		if (size64 > UINT32_MAX) {
			fprintf(stderr, "%s: %" PRIu64 ": %s\n", fname, size64,
				strerror(ERANGE));
			exit(EXIT_FAILURE);
		}
		if ((addr = mmap(NULL, (size_t)size, PROT_READ, MAP_PRIVATE,
				 fd, 0)) == MAP_FAILED) {
			fprintf(stderr, "%s: %s\n", fname, strerror(errno));
			exit(EXIT_FAILURE);
		}

		if (close(fd) == -1) {
			fprintf(stderr, "%s: %s\n", fname, strerror(errno));
			munmap(addr, (size_t)size);
			exit(EXIT_FAILURE);
		}

		if ((rcs = rcslib_init(addr, size)) != NULL) {
			if (quiet) {
				printf("NO ERROR: %s\n", fname);
			} else {
				if (!rcscan_dump(rcs))
					printf("ERROR: %s\n", fname);
			}
		} else {
			printf("ERROR: %s\n", fname);
		}

		if (munmap(addr, (size_t)size) == -1) {
			fprintf(stderr, "%s: %s\n", fname, strerror(errno));
			exit(EXIT_FAILURE);
		}
	}

	exit(EXIT_SUCCESS);
}

boolean
rcscan_dump(struct rcslib_file *rcs)
{
	struct rcsid *id;
	struct rcsnum *num;
	struct rcssym *sym;
	struct iovec iov[10];
	size_t i;
	ssize_t wn;
	int fd = STDOUT_FILENO;

	iov[0].iov_base = "head\t";
	if (rcs->head.n_len > 0)
		iov[0].iov_len = 5;
	else
		iov[0].iov_len = 4;
	iov[1].iov_base = rcs->head.n_str;
	iov[1].iov_len = rcs->head.n_len;
	iov[2].iov_base = ";\n";
	iov[2].iov_len = 2;
	wn = rcs->head.n_len + iov[0].iov_len + 2;
	i = 3;
	if (rcs->branch.n_len > 0) {
		iov[i].iov_base = "branch\t";
		iov[i++].iov_len = 7;
		iov[i].iov_base = rcs->branch.n_str;
		iov[i++].iov_len = rcs->branch.n_len;
		iov[i].iov_base = ";\n";
		iov[i++].iov_len = 2;
		wn += rcs->branch.n_len + 9;
	}
	iov[i].iov_base = "access";
	iov[i++].iov_len = 6;
	wn += 6;
	if (writev(fd, iov, i) != wn) {
		fprintf(stderr, "%s\n", strerror(errno));
		return (FALSE);
	}

	for (i = 0 ; i < rcs->access.ra_count ; i++) {
		id =  &rcs->access.ra_id[i];
		iov[0].iov_base = "\n\t";
		iov[0].iov_len = 2;
		iov[1].iov_base = id->i_id;
		iov[1].iov_len = id->i_len;
		if (writev(fd, iov, 2) != id->i_len + 2) {
			fprintf(stderr, "%s\n", strerror(errno));
			return (FALSE);
		}
	}
	if (write(fd, ";\nsymbols", 9) != 9) {
		fprintf(stderr, "%s\n", strerror(errno));
		return (FALSE);
	}

	for (i = 0 ; i < rcs->symbols.rs_count ; i++) {
		struct rcslib_symbol *symbol = &rcs->symbols.rs_symbols[i];
		sym = &symbol->sym;
		num = &symbol->num;
		iov[0].iov_base = "\n\t";
		iov[0].iov_len = 2;
		iov[1].iov_base = sym->s_sym;
		iov[1].iov_len = sym->s_len;
		iov[2].iov_base = ":";
		iov[2].iov_len = 1;
		iov[3].iov_base = num->n_str;
		iov[3].iov_len = num->n_len;
		if (writev(fd, iov, 4) != sym->s_len + num->n_len + 3) {
			fprintf(stderr, "%s\n", strerror(errno));
			return (FALSE);
		}
	}
	if (write(fd, ";\nlocks", 7) != 7) {
		fprintf(stderr, "%s\n", strerror(errno));
		return (FALSE);
	}

	for (i = 0 ; i < rcs->locks.rl_count ; i++) {
		struct rcslib_lock *lock = &rcs->locks.rl_locks[i];
		id = &lock->id;
		num = &lock->num;
		iov[0].iov_base = "\n\t";
		iov[0].iov_len = 2;
		iov[1].iov_base = id->i_id;
		iov[1].iov_len = id->i_len;
		iov[2].iov_base = ":";
		iov[2].iov_len = 1;
		iov[3].iov_base = num->n_str;
		iov[3].iov_len = num->n_len;
		if (writev(fd, iov, 4) != id->i_len + num->n_len + 3) {
			fprintf(stderr, "%s\n", strerror(errno));
			return (FALSE);
		}
	}

	iov[0].iov_base = ";";
	iov[0].iov_len = 1;
	iov[1].iov_base = " strict;";
	if (rcs->locks.rl_strict)
		iov[1].iov_len = 8;
	else
		iov[1].iov_len = 0;
	iov[2].iov_base = "\ncomment";
	iov[2].iov_len = 8;
	iov[3].iov_base = "\t@";
	if (rcs->comment.s_len > 0)
		iov[3].iov_len = 2;
	else
		iov[3].iov_len = 0;
	iov[4].iov_base = rcs->comment.s_str;
	iov[4].iov_len = rcs->comment.s_len;
	iov[5].iov_base = "@";
	if (rcs->comment.s_len > 0)
		iov[5].iov_len = 1;
	else
		iov[5].iov_len = 0;
	iov[6].iov_base = ";\nexpand\t@";
	if (rcs->expand.s_len > 0)
		iov[6].iov_len = 10;
	else
		iov[6].iov_len = 2;
	iov[7].iov_base = rcs->expand.s_str;
	iov[7].iov_len = rcs->expand.s_len;
	iov[8].iov_base = "@;\n";
	if (rcs->expand.s_len > 0)
		iov[8].iov_len = 3;
	else
		iov[8].iov_len = 0;
	iov[9].iov_base = "\n\n\n";
	iov[9].iov_len = 3;
	wn = iov[1].iov_len + iov[3].iov_len + rcs->comment.s_len +
	     iov[5].iov_len + iov[6].iov_len + rcs->expand.s_len +
	     iov[8].iov_len + 12;

	if (writev(fd, iov, 10) != wn) {
		fprintf(stderr, "%s\n", strerror(errno));
		return (FALSE);
	}

	for (i = 0 ; i < rcs->delta.rd_count ; i++) {
		if (!rcslib_write_delta(fd, &rcs->delta.rd_rev[i]))
			return (FALSE);
	}

	iov[0].iov_base = "\ndesc\n@",
	iov[0].iov_len = 7;
	iov[1].iov_base = rcs->desc.s_str;
	iov[1].iov_len = rcs->desc.s_len;
	iov[2].iov_base = "@\n";
	iov[2].iov_len = 2;
	if (writev(fd, iov, 3) != rcs->desc.s_len + 9) {
		fprintf(stderr, "%s\n", strerror(errno));
		return (FALSE);
	}

	for (i = 0 ; i < rcs->delta.rd_count ; i++) {
		if (!rcslib_write_deltatext(fd, &rcs->delta.rd_rev[i]))
			return (FALSE);
	}

	rcslib_destroy(rcs);

	return (TRUE);
}

void
usage(void)
{
	fprintf(stderr, "Usage: rcscan [-q] <file> ...\n");
	exit(EXIT_FAILURE);
}

==> cvsync-0.24.18.tar.gz: cvsync-0.24.18/rcscan/main.c <==
/*-
 * Copyright (c) 2000-2004 MAEKAWA Masahide <maekawa@cvsync.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <sys/types.h>
#include <sys/mman.h>
#include <sys/uio.h>

#include <stdio.h>
#include <stdlib.h>

#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#include "compat_stdbool.h"
#include "compat_stdint.h"
#include "compat_inttypes.h"

#include "rcslib.h"

bool rcscan_dump(struct rcslib_file *);
void usage(void);

int
main(int argc, char *argv[])
{
	const char *fname;
	struct rcslib_file *rcs;
	void *addr;
	off_t size;
	uint64_t size64;
	int fd, ch;
	bool quiet = false;

	while ((ch = getopt(argc, argv, "q")) != -1) {
		switch (ch) {
		case 'q':
			if (quiet) {
				usage();
				/* NOTREACHED */
			}
			quiet = true;
			break;
		default:
			usage();
			/* NOTREACHED */
		}
	}
	argc -= optind;
	argv += optind;

	if (argc == 0) {
		usage();
		/* NOTREACHED */
	}

	while (argc > 0) {
		fname = argv[--argc];

		if ((fd = open(fname, O_RDONLY, 0)) == -1) {
			fprintf(stderr, "%s: %s\n", fname, strerror(errno));
			exit(EXIT_FAILURE);
		}

		if ((size = lseek(fd, 0, SEEK_END)) == (off_t)-1) {
			fprintf(stderr, "%s: %s\n", fname, strerror(errno));
			close(fd);
			exit(EXIT_FAILURE);
		}
		if (size == 0) {
			fprintf(stderr, "%s: empty file\n", fname);
			close(fd);
			exit(EXIT_FAILURE);
		}
		size64 = (uint64_t)size;
		if (size64 > SIZE_MAX) {
			fprintf(stderr, "%s: %" PRIu64 ": %s\n", fname, size64,
				strerror(ERANGE));
			close(fd);
			exit(EXIT_FAILURE);
		}
		if ((addr = mmap(NULL, (size_t)size, PROT_READ, MAP_PRIVATE,
				 fd, 0)) == MAP_FAILED) {
			fprintf(stderr, "%s: %s\n", fname, strerror(errno));
			close(fd);
			exit(EXIT_FAILURE);
		}

		if (close(fd) == -1) {
			fprintf(stderr, "%s: %s\n", fname, strerror(errno));
			munmap(addr, (size_t)size);
			exit(EXIT_FAILURE);
		}

		if ((rcs = rcslib_init(addr, size)) != NULL) {
			if (quiet) {
				printf("NO ERROR: %s\n", fname);
			} else {
				if (!rcscan_dump(rcs))
					printf("ERROR: %s\n", fname);
			}
			rcslib_destroy(rcs);
		} else {
			printf("ERROR: %s\n", fname);
		}

		if (munmap(addr, (size_t)size) == -1) {
			fprintf(stderr, "%s: %s\n", fname, strerror(errno));
			exit(EXIT_FAILURE);
		}
	}

	exit(EXIT_SUCCESS);
}

bool
rcscan_dump(struct rcslib_file *rcs)
{
	struct rcsid *id;
	struct rcsnum *num;
	struct rcssym *sym;
	struct iovec iov[10];
	size_t i;
	ssize_t wn;
	int fd = STDOUT_FILENO;

	iov[0].iov_base = "head\t";
	if (rcs->head.n_len > 0)
		iov[0].iov_len = 5;
	else
		iov[0].iov_len = 4;
	iov[1].iov_base = rcs->head.n_str;
	iov[1].iov_len = rcs->head.n_len;
	iov[2].iov_base = ";\n";
	iov[2].iov_len = 2;
	wn = rcs->head.n_len + iov[0].iov_len + 2;
	i = 3;
	if (rcs->branch.n_len > 0) {
		iov[i].iov_base = "branch\t";
		iov[i++].iov_len = 7;
		iov[i].iov_base = rcs->branch.n_str;
		iov[i++].iov_len = rcs->branch.n_len;
		iov[i].iov_base = ";\n";
		iov[i++].iov_len = 2;
		wn += rcs->branch.n_len + 9;
	}
	iov[i].iov_base = "access";
	iov[i++].iov_len = 6;
	wn += 6;
	if (writev(fd, iov, i) != wn) {
		fprintf(stderr, "%s\n", strerror(errno));
		return (false);
	}

	for (i = 0 ; i < rcs->access.ra_count ; i++) {
		id =  &rcs->access.ra_id[i];
		iov[0].iov_base = "\n\t";
		iov[0].iov_len = 2;
		iov[1].iov_base = id->i_id;
		iov[1].iov_len = id->i_len;
		if (writev(fd, iov, 2) != id->i_len + 2) {
			fprintf(stderr, "%s\n", strerror(errno));
			return (false);
		}
	}
	if (write(fd, ";\nsymbols", 9) != 9) {
		fprintf(stderr, "%s\n", strerror(errno));
		return (false);
	}

	for (i = 0 ; i < rcs->symbols.rs_count ; i++) {
		struct rcslib_symbol *symbol = &rcs->symbols.rs_symbols[i];
		sym = &symbol->sym;
		num = &symbol->num;
		iov[0].iov_base = "\n\t";
		iov[0].iov_len = 2;
		iov[1].iov_base = sym->s_sym;
		iov[1].iov_len = sym->s_len;
		iov[2].iov_base = ":";
		iov[2].iov_len = 1;
		iov[3].iov_base = num->n_str;
		iov[3].iov_len = num->n_len;
		if (writev(fd, iov, 4) != sym->s_len + num->n_len + 3) {
			fprintf(stderr, "%s\n", strerror(errno));
			return (false);
		}
	}
	if (write(fd, ";\nlocks", 7) != 7) {
		fprintf(stderr, "%s\n", strerror(errno));
		return (false);
	}

	for (i = 0 ; i < rcs->locks.rl_count ; i++) {
		struct rcslib_lock *lock = &rcs->locks.rl_locks[i];
		id = &lock->id;
		num = &lock->num;
		iov[0].iov_base = "\n\t";
		iov[0].iov_len = 2;
		iov[1].iov_base = id->i_id;
		iov[1].iov_len = id->i_len;
		iov[2].iov_base = ":";
		iov[2].iov_len = 1;
		iov[3].iov_base = num->n_str;
		iov[3].iov_len = num->n_len;
		if (writev(fd, iov, 4) != id->i_len + num->n_len + 3) {
			fprintf(stderr, "%s\n", strerror(errno));
			return (false);
		}
	}

	iov[0].iov_base = ";";
	iov[0].iov_len = 1;
	iov[1].iov_base = " strict;";
	if (rcs->locks.rl_strict)
		iov[1].iov_len = 8;
	else
		iov[1].iov_len = 0;
	iov[2].iov_base = "\ncomment";
	iov[2].iov_len = 8;
	iov[3].iov_base = "\t@";
	if (rcs->comment.s_len > 0)
		iov[3].iov_len = 2;
	else
		iov[3].iov_len = 0;
	iov[4].iov_base = rcs->comment.s_str;
	iov[4].iov_len = rcs->comment.s_len;
	iov[5].iov_base = "@";
	if (rcs->comment.s_len > 0)
		iov[5].iov_len = 1;
	else
		iov[5].iov_len = 0;
	iov[6].iov_base = ";\nexpand\t@";
	if (rcs->expand.s_len > 0)
		iov[6].iov_len = 10;
	else
		iov[6].iov_len = 2;
	iov[7].iov_base = rcs->expand.s_str;
	iov[7].iov_len = rcs->expand.s_len;
	iov[8].iov_base = "@;\n";
	if (rcs->expand.s_len > 0)
		iov[8].iov_len = 3;
	else
		iov[8].iov_len = 0;
	iov[9].iov_base = "\n\n\n";
	iov[9].iov_len = 3;
	wn = iov[1].iov_len + iov[3].iov_len + rcs->comment.s_len +
	     iov[5].iov_len + iov[6].iov_len + rcs->expand.s_len +
	     iov[8].iov_len + 12;

	if (writev(fd, iov, 10) != wn) {
		fprintf(stderr, "%s\n", strerror(errno));
		return (false);
	}

	for (i = 0 ; i < rcs->delta.rd_count ; i++) {
		if (!rcslib_write_delta(fd, &rcs->delta.rd_rev[i]))
			return (false);
	}

	iov[0].iov_base = "\ndesc\n@",
	iov[0].iov_len = 7;
	iov[1].iov_base = rcs->desc.s_str;
	iov[1].iov_len = rcs->desc.s_len;
	iov[2].iov_base = "@\n";
	iov[2].iov_len = 2;
	if (writev(fd, iov, 3) != rcs->desc.s_len + 9) {
		fprintf(stderr, "%s\n", strerror(errno));
		return (false);
	}

	for (i = 0 ; i < rcs->delta.rd_count ; i++) {
		if (!rcslib_write_deltatext(fd, &rcs->delta.rd_rev[i]))
			return (false);
	}

	return (true);
}

void
usage(void)
{
	fprintf(stderr, "Usage: rcscan [-q] <file> ...\n");
	exit(EXIT_FAILURE);
}
